﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.MvcEdition;

namespace MapResizing.Controllers
{
    public class HomeController : Controller
    {
        //
        // GET: /Home/

        public ActionResult Index()
        {
            ViewBag.Title = "Map Resizing";

            Map map = SetupMap("Map1", 1550, 700, MappingUtility.DefaultExtent);

            return View(map);
        }

        private Map SetupMap(string mapName, double width, double height, double upperLeftX, double upperLeftY, double lowerRightX, double lowerRightY)
        {
            //Setup ThinkGeo map

            //Setup the map dimensions using System.Web.UI.WebControls units
            System.Web.UI.WebControls.Unit mapWidth = new System.Web.UI.WebControls.Unit(width, System.Web.UI.WebControls.UnitType.Pixel);
            System.Web.UI.WebControls.Unit mapHeight = new System.Web.UI.WebControls.Unit(height, System.Web.UI.WebControls.UnitType.Pixel);

            //Set intitial extent
            RectangleShape initialExtent = new RectangleShape(upperLeftX, upperLeftY, lowerRightX, lowerRightY);

            //Instansiate ThinGeo Map with name, and dimensions
            Map map = new Map(mapName, mapWidth, mapHeight);

            //Set the viewable area, apply projection
            try
            {
                MappingUtility.WGS84toSphMercator.Open();
                map.CurrentExtent = (RectangleShape)MappingUtility.WGS84toSphMercator.ConvertToExternalProjection(initialExtent);
            }
            finally
            {
                MappingUtility.WGS84toSphMercator.Close();
            }

            //Set map unit, use meters since OpenStreetMap uses a spherical mercator projection
            map.MapUnit = GeographyUnit.Meter;

            map.MapBackground = new BackgroundLayer(new GeoSolidBrush(GeoColor.StandardColors.Transparent));

            //Enable and modify any tools that are to be used
            map.MapTools.OverlaySwitcher.Enabled = true;

            //Disable globe buttom that zooms out to whole earth view, look into overriding behavior to recenter on start position; Will probably need to be done clientside via js
            map.MapTools.PanZoomBar.IsGlobeButtonEnabled = false;

            //Setup Overlays
            SetupOverlays(map);

            return map;
        }

        private Map SetupMap(string mapName, double width, double height, RectangleShape initialExtent)
        {
            //Setup ThinkGeo map

            //Setup the map dimensions using System.Web.UI.WebControls units
            System.Web.UI.WebControls.Unit mapWidth = new System.Web.UI.WebControls.Unit(width, System.Web.UI.WebControls.UnitType.Pixel);
            System.Web.UI.WebControls.Unit mapHeight = new System.Web.UI.WebControls.Unit(height, System.Web.UI.WebControls.UnitType.Pixel);

            //Instansiate ThinGeo Map with name, and dimensions
            Map map = new Map(mapName, mapWidth, mapHeight);

            //Set the viewable area, apply projection
            try
            {
                MappingUtility.WGS84toSphMercator.Open();
                map.CurrentExtent = (RectangleShape)MappingUtility.WGS84toSphMercator.ConvertToExternalProjection(initialExtent);
            }
            finally
            {
                MappingUtility.WGS84toSphMercator.Close();
            }

            //Set map unit, use meters since OpenStreetMap uses a spherical mercator projection
            map.MapUnit = GeographyUnit.Meter;

            map.MapBackground = new BackgroundLayer(new GeoSolidBrush(GeoColor.StandardColors.Transparent));

            //Enable and modify any tools that are to be used
            map.MapTools.OverlaySwitcher.Enabled = true;

            //Disable globe buttom that zooms out to whole earth view, look into overriding behavior to recenter on start position; Will probably need to be done clientside via js
            map.MapTools.PanZoomBar.IsGlobeButtonEnabled = false;

            //Setup Overlays
            SetupOverlays(map);

            return map;
        }

        private void SetupOverlays(Map map)
        {
            //Map Suite features a built in overlay for OpenStreetMap data, use this as the base overlay for the map
            map.CustomOverlays.Add(new OpenStreetMapOverlay("OpenStreetMap"));
        }

        [MapActionFilter]
        public JsonResult GetDefaultExtent(Map map, GeoCollection<object> args)
        {
            double[] extent = MappingUtility.GetArrayForProjectedDefaultExtent(MappingUtility.WGS84toSphMercator);

            return Json(new { extent }, JsonRequestBehavior.AllowGet);
        }
    }
}
